﻿using System;
using System.Collections.Generic;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Net.Mime;
using System.Security.AccessControl;
using System.Text;
using System.Threading.Tasks;
using Microsoft.WindowsAzure.Jobs;
using WebMatrix.Data;
using System.Drawing;

namespace ImageUploadAndProcessingJob
{
    internal class Program
    {
        private static string ImageFolderPath
        {
            get { return Environment.ExpandEnvironmentVariables(@"%HOME%\site\wwwroot\App_Data\Upload"); }
        }


        private static void Main(string[] args)
        {
            var host = new JobHost();
            host.RunAndBlock();
        }

        public static void ProcessUploadQueueMessages([QueueInput("uploadqueue")] string queueMessage, IBinder binder)
        {
            var splited = queueMessage.Split(',').Select(m => m.Trim()).ToArray();
            var userName = splited[0];
            var galleryId = splited[1];
            var imageId = splited[2];
            var extension = splited[3];
            var filePath = Path.Combine(ImageFolderPath, imageId + extension);

            UploadFullImage(filePath, imageId + extension, binder);
            UploadThumbnail(filePath, imageId + extension, binder);
            SafeGuard(() => File.Delete(filePath));

            using (var db = Database.Open("PhotoGallery"))
            {
                db.Execute(@"INSERT INTO Photos
                        (Id, GalleryId, UserName, Description, FileTitle, FileExtension, UploadDate, Likes) VALUES 
                        (@0, @1, @2, @3, @4, @5, @6, @7)", imageId, galleryId, userName, "", imageId, extension,
                    DateTime.UtcNow, 0);
            }
        }

        private static void SafeGuard(Action action)
        {
            try
            {
                action.Invoke();
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
            }
        }

        public static void ProcessLikeQueueMessages([QueueInput("likequeue")] string queueMessage)
        {
            var splited = queueMessage.Split(',').Select(m => m.Trim()).ToArray();
            var userName = splited[0];
            var imageId = splited[1];

            using (var db = Database.Open("PhotoGallery"))
            {
                db.Execute("UPDATE Photos SET Likes = Likes + 1 WHERE Id = @0", imageId);
                db.Execute("UPDATE GuidUsers SET TotalLikes = TotalLikes + 1 WHERE UserName = @0", userName);
            }
        }

        private static void UploadFullImage(string imagePath, string blobName, IBinder binder)
        {
            using (var fileStream = new FileStream(imagePath, FileMode.Open))
            {
                using (var outputStream =
                    binder.Bind<Stream>(new BlobOutputAttribute(String.Format("full/{0}", blobName))))
                {
                    fileStream.CopyTo(outputStream);
                }
            }
            RemoteStorageManager.ReplicateBlob("full", blobName);
        }

        private static void UploadThumbnail(string imagePath, string blobName, IBinder binder)
        {
            using (var outputStream =
                binder.Bind<Stream>(new BlobOutputAttribute(String.Format("thumbnail/{0}", blobName))))
            {
                using (var image = Image.FromFile(imagePath))
                {
                    var ratio = Min(200.0/image.Width, 200.0/image.Height);
                    var thumb = image.GetThumbnailImage((int) (image.Width*ratio), (int) (image.Height*ratio),
                        () => false, IntPtr.Zero);
                    thumb.Save(outputStream, ImageFormat.Jpeg);
                }
            }
            System.Threading.Thread.Sleep(1000);
            RemoteStorageManager.ReplicateBlob("thumbnail", blobName);
        }

        private static double Min(double a, double b)
        {
            return a < b ? a : b;
        }
    }
}
